<?php

namespace Drupal\mason\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\blazy\Plugin\Field\FieldFormatter\BlazyFileFormatterBase;
use Drupal\mason\MasonDefault;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for mason image and file ER formatters.
 *
 * @todo extends BlazyFormatterBlazy post blazy:2.17 instead.
 */
abstract class MasonFileFormatterBase extends BlazyFileFormatterBase {

  use MasonFormatterTrait;

  /**
   * {@inheritdoc}
   */
  protected static $namespace = 'mason';

  /**
   * {@inheritdoc}
   */
  protected static $itemId = 'box';

  /**
   * {@inheritdoc}
   */
  protected static $captionId = 'caption';

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    return self::injectServices($instance, $container, 'image');
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return MasonDefault::imageSettings();
  }

  /**
   * {@inheritdoc}
   *
   * @todo remove post blazy:2.17, no real difference this far.
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $entities = $this->getEntitiesToView($items, $langcode);

    // Early opt-out if the field is empty.
    if (empty($entities)) {
      return [];
    }

    return $this->commonViewElements($items, $langcode, $entities);
  }

  /**
   * {@inheritdoc}
   *
   * @todo remove post blazy:2.17, no real difference this far.
   */
  public function buildElements(array &$build, $files, $langcode) {
    foreach ($this->getElements($build, $files) as $element) {
      $build['items'][] = $element;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    if (isset($element['image_style'])) {
      $element['image_style']['#description'] = $this->t('This will be treated as the fallback image style if the optionset image styles not provided.');
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function getScopedFormElements() {
    return [
      'background'  => TRUE,
      'breakpoints' => FALSE,
      'no_ratio'    => TRUE,
    ] + parent::getScopedFormElements();
  }

}
